# Computing a reference solution with fenics

from dolfin import *
import math
import matplotlib.pyplot as plt

# Configuration (see mesher.py)
centres = [0., 2./5.]
rayons = [1., 2./5.]
mesh_path = "mesh/couronne.3.xml"

# Mesh and solution space
mesh_1 = Mesh(mesh_path)
V_1 = FunctionSpace(mesh_1, 'Lagrange', 2)
coords = mesh_1.coordinates()
xx = [e[0] for e in coords]
yy = [e[1] for e in coords]

# Boundary conditions
u_outer = Constant(0.0)
u_inner = Constant(1.0)

def cl_outer(x, on_boundary):
    cx = centres[0].real
    cy = centres[0].imag
    tol = 1e-3
    return on_boundary and \
        near((x[0]-cx)*(x[0]-cx)+(x[1]-cy)*(x[1]-cy), rayons[0], tol)

def cl_inner(x, on_boundary):
    cx = centres[0].real
    cy = centres[0].imag
    tol = 1e-2
    return on_boundary and \
        ((x[0]-cx)*(x[0]-cx)+(x[1]-cy)*(x[1]-cy) < rayons[0]-tol)

bcs = [DirichletBC(V_1, u_outer, cl_outer),
       DirichletBC(V_1, u_inner, cl_inner)]

# Variational problem
u_1 = TrialFunction(V_1)
v = TestFunction(V_1)

f = Constant(0.0)

a = dot(grad(u_1), grad(v))*dx
L = f*v*dx

# Solution
u_1 = Function(V_1)
solve(a == L, u_1, bcs)

# Function at mesh nodes
vals = u_1.compute_vertex_values(mesh_1)
for x,y, v in zip(xx, yy, vals):
    print x,y, v

# Plot solution
sc = plt.scatter(xx, yy, c=vals)
plt.colorbar(sc)
plt.gca().set_aspect('equal', adjustable='box')
plt.show()
